<#
Innlesing av fil fra Biztalk inn til vår modell

Eks på initiering av modell samt kjøring av progam:

$b = new-object collections.arraylist

Husk å tilpass to stier(parseBinding og biztalkEksempelFil) i linjen nedenfor
$b =  C:\Users\Mathi\OneDrive\Desktop\Bachelorkode\MOI-NY\MOI-NY\MOI-AUTOMATIC\ParseBinding.ps1 -fileName "C:\Users\Mathi\OneDrive\Desktop\Bachelorkode\MOI-NY\MOI-NY\MOI-AUTOMATIC\biztalkEksempelFil.xml" -currentModel $b

For å inspisere modellen:

$b
Viser alle informasjon som blir hentet inn i sesjonsvariabelen b

$b.sendPortCollectionName
Viser alle send porter knyttet til den valgte integrasjonen (biztalkEksempelFil)

$b.sendPortCollectionName.Count
Viser antall sendport-lokasjoner

$b.receivePortCollectionName
viser alle receive porter som er knyttet til den valgte integrasjonen (biztalkEksempelFil)

$b.receivePortCollectionName.Count
Viser antall receiveport-lokasjoner


#>

Param(
    [Parameter(HelpMessage="Input fil")][string] $fileName
    ,[Parameter(Mandatory=$true,HelpMessage="Gjeldende model")][AllowEmptyCollection()][collections.arraylist]$currentModel
)

# Import av fellesmodul. 
. "C:\Users\Mathi\OneDrive\Desktop\Bachelorkode\MOI-NY\MOI-NY\MOI-AUTOMATIC\felles.ps1" #Husk å tilpass stien her


#$mainArray = $currentModel

if($fileName.Length -gt 0) {
    Write-Host "Filnavn " $fileName
    [xml]$xml = Get-Content $fileName
} else {
    Write-Host "Filnavn mangler -avslutt"
    exit
}


[xml]$bindingFile = [xml](Get-Content -Path "C:\Users\Mathi\OneDrive\Desktop\Bachelorkode\MOI-NY\MOI-NY\MOI-AUTOMATIC\biztalkEksempelFil.xml") #Husk å tilpass stien her


$mainid = $bindingFile.BindingInfo.ModuleRefCollection.ModuleRef.Name[1]

    
    $tmpName = $bindingFile.BindingInfo.ModuleRefCollection.ModuleRef.Name[0]
    Write-Host "Module: "$tmpName  # Application Component
    #Stripper vekk første del, eks: [Application:AO.GAT_ORBIT-RodePrikker.HVN]
    if($tmpName.StartsWith('[Application:')) {
        $tmpName = $tmpName.Substring(13)
        $tmpName = $tmpName.Substring(0,$tmpName.IndexOf(']'))
        Write-Host " ModuleRef Fant biztalk Application: " $tmpName -BackgroundColor Green
        # Her har vi da altså funnet navnet...
        
    }


# Sjekk om denne er i modellen
$mainObject = GetObjekt $mainId $currentModel 

if($null -eq $mainObject) { # Vi fant den ikke i modellen - opprett
    $mainObject = AddGetObjekt $mainId $currentModel 
    $mainObject.id = $mainId
    $mainObject.name = $xml.item.name
    $mainObject.SRC = "BINDING"
    $mainObject.SRCref = $fileName
    $mainObject.product = "INTEGRASJONER" # Henter inn product fra Asssyt til tilsvarende integrasjon
    $mainObject.productClass = "BIZTALK APPLIKASJONER" # Henter inn productClass fra Asssyt til tilsvarende integrasjon
    $mainObject.genericClass = "INTEGRASJONER" # Henter inn genericClass fra Asssyt til tilsvarende integrasjon
    $mainObject.dependencyDiscovery = $true # Ja - denne har vi sjekket for avhengigheter


    # Send og Recieveport i binding file
    $mainObject.sendPortCollectionName = $xml.BindingInfo.SendPortCollection.SendPort.Name
    $mainObject.receivePortCollectionName = $xml.BindingInfo.ReceivePortCollection.receivePort.Name

    # Legg til property - som et eksempel
    $mainObject.Properties["AssystID"]          = $systemID
    $mainObject.Properties["productId"]         = $xml.item.productId
    $mainObject.Properties["productType"]       = $xml.item.product.productType  # Denne er som regel $null i hoved-xml
    $mainObject.Properties["productTypeEnum"]   = $xml.item.product.productTypeEnum  #...det samme her
} else {
    Write-Host "Objektet finnes i modellen : "$mainId
}
$mainObject.dependencyDiscovery = $true # Ja - denne har vi nå sjekket for avhengigheter siden dette er hovedobjektet i/fra assyst

# Så går vi gjennom relasjoner

#$relArray = @($xml.item.relations)  # husk at dette kan være 1..n - altså sier vi det skal være et array (casting av variabel)

$relArray = @($xml.BindingInfo.SendPortCollection.SendPort.Name)  # husk at dette kan være 1..n - altså sier vi det skal være et array (casting av variabel)

if($null -ne $relArray) {
    foreach( $relasjon in $relArray) {

        # For hver 'relation'
        Write-Host "Behandler relation (ID): "$relasjon.id 
        $doThis = $true  # Hjelpevariabel for å holde orden på om vi skal legge til en relasjon eller ikke
        $relatedDetail = $null
        try {
            $relatedDetail = $relasjon.relatedDetail # relatedDetail sier hvilken relasjon dette er
        }
        catch {
            $doThis = $false
            Write-Host "-- Har related tag, ingen relatedDetail"
        }
        #$doThis = $true
        # Sjekk også <discontinued>true</discontinued>
        if($relasjon.discontinued -eq "true") {
            $doThis = $false
            Write-Host " >Discontinued="$relasjon.discontinued 
        }
        if($true -eq $doThis) {  # Hvis ikke discontinued
            if($null -eq $relatedDetail) { # Det finnes faktisk ingen relasjon
                $doThis = $false
                Write-Host " > Ingen relatedDetail." -NoNewline
            }
        }

        # Det kan hende det allerede finnes en relasjon fra objektet i modellen - hvis det gjør det - ikke legg til en ny
        if($doThis -eq $true) {                         # Vi tenker vi egentlig skal legge til en relasjon
            $relatedItem = $relasjon.relatedItem        # Dette er den vi har relasjon mot
            foreach($sjekk in $mainObject.Related) {
                if($sjekk.RelatedItemId -eq $relatedItem.shortCode) {
                    $doThis = $false
                    Write-Host " > DUPLIKAT" -NoNewline -ForegroundColor Gray
                }
            }   
        }

        if($doThis -eq $true) {                         # Fremdeles tenker vi å legge til en relasjon
            if("true" -eq $relatedItem.discontinued) { ##### objekt markert som discontinued, men relasjon er aktiv
                Write-Host " > "$newObjectId" er markert som DISCONINUED." -BackgroundColor Red
                $doThis = $false
            } 
        }
        
        # 
        if($true -eq $doThis) { 
            # Vi har en relasjon, så da kan vi etter mye sjekking opprette objektet 
            # Opprett selve relasjonsobjektet - som skal legges til hovedobjektet
            $relationO = createRelationObject

            # Vi legger til dette nye objektet til vår samling av objekter (om det trengs)
            # NB dette handler om det objektet som er i 'andre enden' av relasjonen 
            #$localName = $relatedItem.shortCode
            $newObjectId = $relatedItem.shortCode.ToUpper()
            # newObject er objektet det skal relateres TIL - altså opprett hvis det ikke finnes, ellers bruk eksisterende i modellen.
            $newObject = GetObjekt $newObjectId $currentModel # Finnes objektet allerede?

            # Vi fant ikke objektet i modellen - lag et nytt
            if($null -eq $newObject) {
                $newObject = AddGetObjekt $newObjectId $currentModel 
                # Sett properties på det nye objektet
                $newObject.name = $relatedItem.name
                $newObject.SRC = "ASSYST"
                $newObject.SRCref = $fileName

                # Produkthierarki
                $newObject.product         = $relatedItem.product.shortCode
                $newObject.productClass    = $relatedItem.product.productClass.shortCode
                $newObject.genericClass    = $relatedItem.product.productClass.genericClass.shortCode

                #if($true -eq $newObject.dependencyDiscovery)  Hva om vi henter opp et objekt som finnes fra tidligere ? resolvedBySource er true...
                $newObject.dependencyDiscovery = $false # relasjoner ligger ikke videre her....
            } 
            
            # Selve avhengigheten - igjen så er det opp til dere om dere vil bruke kodeverdier, mappinger etc.  Dette er bare en enkel måte å gjøre det på
            $relationO.RelatedType       = $relatedDetail.shortCode  # <shortCode>B Avhengig av</shortCode>
            $relationO.RelatedName       = $relatedDetail.name       # <name>B ALVORLIG</name>
            $relationO.RelatedItemId     = $newObject.id       # Hvem relasjonen er MOT

            # Vi legger til relasjonen - denne funksjonen vil også sjekke om det allerede finnes en relasjon.
            AddRelation $mainObject $relationO $currentModel $fileName

            #----------------------  Eksempel-del på relasjon tilbake -------------------------
            # Det bør jo også legges til relasjoner 'andre veien' også - dvs. Nødvendig for gir jo implisit at andre veien gjelder 'Avhengig av'
            # DENNE DELEN ER KUN MENT SOM ET EKSEMPEL.
            # Det finnes flere relasjonstyper - ikke kun disse som er listet her
            $tilbakeType = ""
            switch ($relatedDetail.shortCode.ToUpper()) {  # Som et eksempel
                "A NØDVENDIG FOR" { $tilbakeType = "A AVHENGIG AV" }
                "B NØDVENDIG FOR" { $tilbakeType = "A AVHENGIG AV" }
                "C NØDVENDIG FOR" { $tilbakeType = "A AVHENGIG AV" }
                Default {$tilbakeType = "Ukjent"}
            }
            $relationO = createRelationObject
            $relationO.RelatedType       = $tilbakeType  # <shortCode>B Avhengig av</shortCode>
            $relationO.RelatedName       = $relatedDetail.name       # <name>B ALVORLIG</name>
            $relationO.RelatedItemId     = $mainObject.id       # Hvem relasjonen er MOT
            AddRelation $newObject $relationO $currentModel $fileName
            #----------------------  END Eksempel-del på relasjon tilbake -------------------------

        }
    }
}
return $currentModel


Exit